<?php

namespace App\Console\Commands;

use App\Models\BimFile;
use App\Models\Enums\BimFileConvertStatus;
use App\Services\File\BIM\BIMDriverEnum;
use App\Services\File\BIM\BIMFactory;
use Carbon\Carbon;
use Illuminate\Console\Command;
use Illuminate\Database\Eloquent\Collection;

class AddBimToConvertQueue extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'lpc:add-bim-to-convert-queue';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Timed push of bim files to conversion queue';

    /**
     * Execute the console command.
     */
    public function handle()
    {
        BimFile::query()
            ->where("created_at", ">=", Carbon::now()->subDay())
            ->whereIn("convert_status", [BimFileConvertStatus::NotAdded->value, BimFileConvertStatus::FAILED_TO_ADD_QUEUE->value])
            ->where("bim_driver", BIMDriverEnum::BLACK_HOLE->value)
            ->chunkById(2000, function (Collection $bimFiles) {
                $this->pushToQueue($bimFiles);
            });
    }

    protected function pushToQueue(Collection $bimFiles)
    {
        foreach ($bimFiles as $bimFile) {
            try {
                BIMFactory::make($bimFile->bim_driver)->addToConvertQueue($bimFile->bim_data_set_id);
                $bimFile->convert_status = BimFileConvertStatus::IN_QUEUE;
            } catch (\Exception $exception) {
                $bimFile->convert_status = BimFileConvertStatus::FAILED_TO_ADD_QUEUE;
                $bimFile->error = $exception->getMessage();
            }

            $bimFile->save();
        }
    }
}
