<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Requests\API\AssetGroup\CreateOrUpdateRequest;
use App\Http\Resources\API\AssetGroupResource;
use App\Models\AssetGroup;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class AssetGroupController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        $pageSize=$request->get('page_size') ?? 10;
        $groups = AssetGroup::filter($request->all())->orderByDesc("sequence")->paginate($pageSize);
        return AssetGroupResource::collection($groups);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(CreateOrUpdateRequest $request)
    {
        AssetGroup::create([
            ...$request->all(),
            'company_id' => Auth::user()->company_id,
        ]);

        return $this->created();
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        $asset = AssetGroup::findOrFail($id);

        return new AssetGroupResource($asset);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(CreateOrUpdateRequest $request, string $id)
    {
        $group = AssetGroup::query()->findOrFail($id);

        $group->fill($request->only(
            'name', 'sequence'
        ));
        $group->save();

        return $this->noContent();
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        $group = AssetGroup::query()->findOrFail($id);
        if(($group->assets)->isEmpty()){
            $group->delete();
        }
        else{
            throw new \Exception("The asset group still has assets associated with it, and cannot be deleted.");
        }

        return $this->noContent();
    }

}
