<?php

namespace App\Http\Requests\API\Plan;

use App\Http\Requests\RuleHelper;
use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Validation\Rule;

class CreateOrUpdateRequest extends FormRequest
{
    use RuleHelper;

    /**
     * Determine if the user is authorized to make this request.
     */
    public function authorize(): bool
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, \Illuminate\Contracts\Validation\ValidationRule|array<mixed>|string>
     */
    public function rules(): array
    {
        return [
            'asset_id' => [
                'required',
                Rule::exists('assets', 'id')->where($this->userCompanyWhere()),
            ],
            'title' => 'required|max:255',
            'begin' => 'date',
            'end' => 'date',
            'parent_id' => [
                $this->parentIdExistsRule(),
            ]

        ];
    }

    protected function parentIdExistsRule()
    {
        // 如果 parent_id 不为 0，返回 exists 规则
        if ($this->input('parent_id') != 0) {
            return Rule::exists('plans', 'id')->where($this->userCompanyWhere())->where('asset_id', $this->input('asset_id'));
        }

        // 如果 parent_id 为 0，返回空数组以跳过 exists 验证
        return [];
    }

}
