<?php

namespace App\Models;

use App\Models\Scopes\CompanyScope;
use EloquentFilter\Filterable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Container extends Model
{
    use HasFactory, SoftDeletes, Filterable;

    protected $fillable = [
        'name', 'library_id', 'naming_rule_id', 'naming_rules', 'mailto', 'email_subject', 'acl', 'whitelist',
        'doc_stage', 'doc_type'
    ];

    protected $casts = [
        'mailto' => 'array',
        'whitelist' => 'array',
        'naming_rules' => 'array',
    ];

    protected static function booted(): void
    {
        static::addGlobalScope(new CompanyScope);
    }

    public function scopeAllowed(Builder $query, string $id = null)
    {

    }

    public function library(): \Illuminate\Database\Eloquent\Relations\BelongsTo
    {
        return $this->belongsTo(Library::class, "library_id");
    }

    public function namingRule(): \Illuminate\Database\Eloquent\Relations\BelongsTo
    {
        return $this->belongsTo(NamingRule::class);
    }

    public function createdBy(): \Illuminate\Database\Eloquent\Relations\BelongsTo
    {
        return $this->belongsTo(User::class, 'created_by');
    }

    public function content(int $version = 0): \Illuminate\Database\Eloquent\Relations\HasOne
    {
        $version = $version > 0 ? $version : $this->version;

        return $this->hasOne(ContainerContent::class, "container_id")->where("version", $version);
    }
}
