<?php

namespace App\Services\History\Detector;

use App\Services\History\Converter\CustomFieldSelectConverter;
use App\Services\History\Converter\EmailConverter;
use App\Services\History\Converter\ModelEnumConverter;
use App\Services\History\Converter\NamingRuleConverter;
use App\Services\History\Converter\ProjectConverter;
use App\Services\History\Converter\RequirementGroupConverter;
use App\Services\History\Converter\TaskConverter;
use App\Services\History\Converter\UserConverter;
use App\Services\History\Converter\WhitelistConverter;

class TaskDetector extends DetectorAbstract
{
    public static function fields(): array
    {
        return [
            'name',
            'project_id',
            'requirement_id',
            'naming_rule_id',
            'parent_id',
            'task_type',
            'doc_stage',
            'status',
            'acl',
            'whitelist',
            'description',
            'mailto',
            'email_subject',
            'begin',
            'end',
            'closed_by',
            'canceled_by',
            'approve_by',
            'finished_by',
            'review_by',
            'created_by',
            'suitability',
            'state',
            'asset_group_id',
        ];
    }

    public static function diffFields(): array
    {
        return [
            'description',
        ];
    }

    public static function arrayFields(): array
    {
        return [
            'mailto',
        ];
    }

    public static function converters(): array
    {
        return [
            "whitelist" => new WhitelistConverter(),
            "acl" => new ModelEnumConverter("project.acl"),
            "status" => new ModelEnumConverter("project.status"),
            "type" => new CustomFieldSelectConverter("project", "type"),
            "requirement_group_id" => new RequirementGroupConverter(),
            "project_id" => new ProjectConverter(),
            "reviewed_by" => new UserConverter(),
            'closed_by' => new UserConverter(),
            'canceled_by' => new UserConverter(),
            'approve_by' => new UserConverter(),
            'finished_by' => new UserConverter(),
            'review_by' => new UserConverter(),
            "state" => new CustomFieldSelectConverter("task", "state"),
            "task_type" => new CustomFieldSelectConverter("task", "task_type"),
            "doc_stage" => new CustomFieldSelectConverter("task", "doc_stage"),
            "naming_rule_id" => new NamingRuleConverter(),
            "parent_id" => new TaskConverter(),
            "mailto" => new EmailConverter(),
        ];
    }
}
